use strict;
use warnings;
use lib do { __FILE__ =~ m|^(.*?)[^/]*$|; "$1"; };

sub sendmail {

    my ($msgfile, $subj, $recipient, $attachment) = @_;

    unless(defined $msgfile && defined $subj && defined $recipient) {
	print "Valid message file, subject, and recipient not provided\n";
	print "email notification not sent\n";
	return;
    }

    my $mutt = `which mutt`;
    chomp($mutt);
    if($mutt =~ /command not found/i) {
	print "Cannot find mutt mail program\n";
	print "email notification not sent\n";
	return;
    }

    # Quick test for valid email address. Won't catch all badly formed
    # addresses, but will at least get the low-hanging fruit
    unless(wellformed_email($recipient)) {
	print "Valid email address not provided\n";
	print "email notification not sent\n";
	return;
    }
    $recipient =~ s/\@/\\@/g;

    my $cmd = "$mutt -s '$subj' ";
    if(defined $attachment) {
	$cmd .= "-a $attachment ";
    }
    $cmd .= "$recipient < $msgfile";

    system($cmd);
    return;
}

sub wellformed_email {

    # This routine returns true if the email address is properly
    # formed. As currently written, it is strongly biased towards
    # false positives rather than false negatives and is used
    # essentially as a sanity check on an address. Note that although
    # an address may be properly formed, it does not mean that it
    # necessarily mean that it is a valid address.
    #
    # A small number of valid, but unusual addresses will be
    # rejected. These include addresses with quotes, embedded spaces,
    # and square brackets. For example "a b c"@xyz.com or
    # abc@[120.121.122.123]


    my ($address) = @_;
    my $valid = 1; # Assume valid by default

    # Step 1 - make sure address is of form string1@string2
    unless($address =~ /^(.*)\@(.*)$/) {
	return(0);
    }

    # Step 2 - make sure that each string contains only allowed characters
    my ($string1, $string2) = ($1, $2);
    unless($string1 =~ /^[a-zA-Z0-9_\.\-]+$/ &&
	   $string2 =~ /^[a-zA-Z0-9_\.\-]+$/) {
	return(0);
    }

    # Step 3 - neither string should start or end with '.'
    if($string1 =~ /^\./ || $string1 =~ /\.$/ ||
       $string2 =~ /^\./ || $string2 =~ /\.$/) {
	return(0);
    }

    # If we made it this far, email address is valid
    return(1);
}
1;
