      subroutine read_files(pixels, start_view, 
     +   end_view, dest, orient)
C                                                             READ_FILES
C **********************************************************************
C *                                                               *
C * Subroutine: read_files                                        *
C * Author: Hong Lin (linh@cs.purdue.edu)                         *
C * Date:   1998                                                  *
C * Used in: cmpt_intrps -- The subroutine for Fourier transform  *
C *                         and interpolation                     *
C * Function: Driver for reading pixel frames from an external    *
C *           file.                                               *
C * Usage:                                                        *
C *   call read_files(pixels, cur_view, orient)                   *
C * Input:                                                        *
C *   cur_view (Integer) -- The current view number               *
C *   orient (Real) -- The orientations and center coordinates of *
C *                    pixel frames                               *
C * Output:                                                       *
C *   pixels (Real) -- The pixel frame in real type.              *
C *                                                               *
C * Algorithm:                                                    *
C *   Variable CUR_VIEW marks the next pixel frame to be read for *
C *   processing. This subroutine reads NUM_NODES pixel frames    *
C *   beginning from CUR_VIEW and scatters them to other nodes so *
C *   that each node will hold one pixel frame to process.        *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 2.0, Oct. 8, 1999                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C *                                                               *
C *****************************************************************
C
      implicit none

      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      INCLUDE '../include/pifhead.inc'
Cji      RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD
Cji  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
      type (PIFHEAD_T) MAPHEAD, IMGHEAD
Cji  added by ji for SP
Cji  and modify maphead.XX to maphead%XXX
Cji  and modify imghead.XX to imghead%XXX
Cji  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD
C
C
      integer*4 start_view, end_view, dest
      real*4  pixels(npix_row, npix_col, view_perread)
      real*4  orient(0:9, max_views)
C --------------------------------------------------------------------
C Local
C 
      integer*4 itemp, view, stat(MPI_STATUS_SIZE)
      character*10 subname
      data subname / 'read_files' /
C
C --------------------------------------------------------------------
C
C Beginning of first main loop: read and process a set of pixel frames 
C
      itemp  = npix_row*npix_col
      if (iam.eq.0) then
        do view = start_view, end_view
          IMGHEAD%IMGNUM = nint(orient(0, view))

          CALL PIF_READ_DH(IMGHEAD)
C
	  IMGHEAD%luNum = pix_file_unit
          call rpifimag_f (itemp,
     1      pixels(1, 1, view - start_view + 1))

         enddo
         if (dest .ne. 0) then
           call mpi_send( pixels, view_perread * itemp, mpi_real,
     +                 dest, dest, mpi_comm_world, mpi_err)
           if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +       'Error in calling SEND pixels')
         endif
      else
        if (iam .eq. dest) then
          call MPI_RECV(pixels, view_perread * itemp, mpi_real,
     +                 0, dest, mpi_comm_world, stat, mpi_err)
          if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +     'Error in calling MPI_RECV pixels')
        endif 
      endif

Cji   end of the read  end_view-start_view+1 pixels
C
      return
      end
