      PROGRAM POR
C **********************************************************************
      implicit none
C
      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
      include '../include/mem_time.inc'
      
      logical :: confine_3D = .false.

C ----------------------------------------------------------------------
C Local variables, used only in the main routine

      real center(2)

      integer first_iq, last_iq, max_iq_slab,
     +     first_z,last_z, first_hk,last_hk, max_hk_slab, num_hkl,
     +     den_min_x, den_max_x, den_min_y, den_max_y, 
     +     den_min_z, den_max_z, mem_size, 
     +     min_xyz, max_xyz, hkl_max, max_z_slab,
     +     first_rr_out, last_rr_out, max_rr_out_slab,
     +     first_u, last_u, max_u_slab,
     +     first_y, last_y, max_y_slab
      
      integer nrffts_1_row, nrffts_1_col, nrffts_1_omega,
     +     nrffts_2_row, nrffts_2_col, nrffts_2_omega
      
      integer alloc_stat, ierr
      
C -----------------------------------------------------------------
C     The next arrays variables have their lenghts defined dynamically

      real, allocatable :: ctf_val(:,:), abs_ctf(:,:), disef(:,:), 
     +     map3d(:,:,:), rffta_omega(:)
      
      complex, allocatable :: fft3d(:), fft_klh(:)
      
      character(len=17) :: subname = 'POR main routine'
      
      REAL   Weight, mapprjbig, imgprjbig, enlargefact
      integer fft_ann
      
      integer i, j, total_views
      
C *********************************************************************
C *                      Initialization                               *
C *********************************************************************

      PROGNAME = 'POR '
      call usage

C     INITIALIZE MPI
C     **************
      call MPI_INIT ( mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_INIT')
      call MPI_COMM_RANK ( MPI_COMM_WORLD, iam, mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_COMM_RANK')
      call MPI_COMM_SIZE ( MPI_COMM_WORLD, num_nodes, mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_COMM_SIZE')
      if (num_nodes > max_numnodes) then
         write(stdoutput, 9001) num_nodes, max_numnodes
 9001    format(//' Requested number of nodes = NUM_NODES    = ',i10/
     +        ' Available number of nodes = MAX_NUMNODES = ',i10/)
         call error_stop(progname, subname,  
     +        'NUM_NODES requested exceeds MAX_NUMNODES')
      endif

      t1 = wtime()

      if(iam == 0) then
         write(*,'(A, i5)') 'Number of Nodes =', num_nodes

C       Read User's Control-Input
Cji     I have change the control_input to multi files.

         call info

Cji   check the map, and get the col and row number
Cji   3D model should have the smallest pixel_size and largest box_size.
Cji   comapred with boxed images.

         call get_map_dimensions(in_3d_name, npix_col, npix_row, 
     $        angstrom_per_pix, ierr)
         if(ierr > 0) then
            call error_stop('POR', 'POR',
     +           'Error in 3D map mode, it should be 20 or 40')
         endif

C     3D map should have larger dimension than image box size
         if(npix_row < box_out) then
            call error_stop(progname, subname, 
     $           'NPIX_ROW is less than BOX_OUT')
         endif
         
C     Check the map pixel size
         if(angstrom_per_pix < 0.01 .or. angstrom_per_pix > 100.0) then
            write(*,*) '3D map has nonsensical pixel size'
            write(*,*) 'Use pixel size from particle parameter files'
            angstrom_per_pix = PIXSIZ_OUT
         elseif ((angstrom_per_pix - PIXSIZ_OUT) > 10E-5) then
            call error_stop(progname, subname, 
     $           'Pixel size from 3D map is too large')
         else
            PIXSIZ_OUT = angstrom_per_pix
         endif
         
      endif

      if(num_nodes > 1) then
C     CALL BCAST_PARAMETERS
C     *******************************
C     Broadcast User's parameters and PIF File Header to other nodes.
         call bcast_parameters 
      endif

C     CALL INTLZ_PARAMS
C     *****************
      call intlz_params(hkl_max, num_hkl, min_xyz, max_xyz, 
     +     nrffta_row, nrffta_col, nrffta_omega, nfc_2d_row, 
     +     nfc_2d_col, nrffts_1_row, nrffts_1_col, 
     +     nrffts_1_omega, nrffts_2_row, nrffts_2_col, 
     +     nrffts_2_omega, den_min_x, den_max_x, den_min_y, 
     +     den_max_y, den_min_z, den_max_z,
     +     confine_3D)

C     ji 01/06/01 added 2 print statements after call of intlz_params
      if(iam == 0) then
         print *, ' zero_fill after  call of INTLZ_PARAMS =',zero_fill
         print *, ' nrffta_row after call of INTLZ_PARAMS =',nrffta_row
      endif

      call intlz_arrays(hkl_max, min_xyz, max_xyz, min_z, max_z,
     +     first_hk, last_hk, max_hk_slab, first_iq, last_iq,
     +     max_iq_slab, first_rr_out, last_rr_out, max_rr_out_slab,
     +     first_u, last_u, max_u_slab,
     +     first_y, last_y, max_y_slab, first_z, last_z, max_z_slab,
     +     npix_row)

C     ---------- End of Initializationl --- Synchronize all the nodes ------

      call mpi_barrier(mpi_comm_world, mpi_err)
      t2 = wtime()
      t_init = t2 - t1
      
C **********************************************************************
C *             3D DFT of input 3D density map                         *
C **********************************************************************
C
      mem_size = npix_col * npix_row * max_y_slab
      allocate(map3d(npix_col, npix_row, max_y_slab), 
     +     stat = alloc_stat)
      mem_map3d = dble(4 * mem_size) 
      if (alloc_stat > 0) call error_stop (progname, subname,
     +     '3D density map memory allocation failed')
      mem_total = mem_total + mem_map3d
      mem_peak  = max(mem_peak, mem_total)
      
      call read_map(in_3d_name, map3d, npix_col, npix_row,
     +     max_ y_slab, mapprjbig)

Cji  3d DFT, and exchange 8 quater

      allocate(rffta_omega(nrffta_omega ), stat = alloc_stat)
      if(alloc_stat > 0) call error_stop (progname, subname,
     +     'Error in allocating memory for nrffta_omega')
      
      call vrffti( nrffta_col, rffta_omega )
      
      mem_size = (hkl_max+1)* num_hkl * (last_y - first_y + 1) 
      allocate(fft3d(mem_size), stat = alloc_stat)
      mem_fft3d = dble(8 * mem_size) 
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D density fft map memory allocation failed')
      endif
      mem_total = mem_total + mem_fft3d
      mem_peak  = max(mem_peak, mem_total)
C     
Cji   FFT3D, (X,Y,Z) real -> (H,K,Z) complex(0:Hmax, Kmin:Kmax, Z_slab Zmin:Zmax)
Cji because I change intlz_params.f to make hkl_max = nfc_2d_row = nfc_2d_col
C
      do i = 0, last_y - first_y
         center(1) = (npix_row-1)/2.0
         center(2) = (npix_col-1)/2.0
         call fft_2dfft(map3d(1,1,i+1), 
     +        npix_row, npix_col, center,
     +        nrffta_omega, rffta_omega, nrffta_row, nrffta_col, 
     +        hkl_max, hkl_max, FFT3D(i*(hkl_max+1)*num_hkl+1))
      enddo

      deallocate(map3d)
      mem_total = mem_total - mem_map3d
      call mpi_barrier(mpi_comm_world, mpi_err)

Cji need exchange data between nodes from Z_slab to H_slab

      if (num_nodes > 1) then
         
         mem_size = (hkl_max+1)*num_hkl*(last_y - first_y + 1)
         allocate(fft_klh(mem_size), stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           '3D density fft exchange memory 1 allocation failed')
         endif
         mem_total = mem_total + 8*mem_size 
         mem_peak  = max(mem_peak, mem_total)
         
         call arrange_3d_1(FFT3D, fft_klh, hkl_max)
         
         deallocate(fft3d)
         mem_total = mem_total - mem_fft3d
         mem_fft3d = 8 * mem_size
         
         mem_size = (last_iq-first_iq+1)*num_hkl*npix_row
         allocate(fft3d(mem_size), stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           '3D density fft exchange memory 2 allocation failed')
         endif
         mem_total = mem_total + 8*mem_size 
         mem_peak  = max(mem_peak, mem_total)
         
         call exch_3d_1(fft_klh, hkl_max, FFT3D, npix_row)

         deallocate(fft_klh)
         mem_total = mem_total - mem_fft3d
         mem_fft3d = 8 * mem_size
      endif

Cji   1D fft, (H,K,Z)-->(K,L,H) complex(Kmin:Kmax, Lmin:Lmax, H_slab 0:Hmax)

      mem_size = (last_iq-first_iq+1)* num_hkl**2
      allocate(fft_klh(mem_size), stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D density fft 1dfft allocation failed')
      endif
      mem_total = mem_total +8*mem_size 
      mem_peak  = max(mem_peak, mem_total)

      CALL cfft_1d(FFT3D, fft_klh, hkl_max, npix_row,
     +     nrffta_omega, rffta_omega)

      deallocate(fft3d)
      mem_total = mem_total - mem_fft3d
      mem_fft3d = 8 * mem_size

Cji apply tmpfac to the slab of map.
CRSS Note new calling sequence for maptempfac
CRSS      if (ctfmode > 0 .and. ctfmode <= 4) then
CRSS         call maptempfac(fft_klh, -hkl_max, hkl_max, -hkl_max, hkl_max,
CRSS     $        frst_lst_iq(1,iam), frst_lst_iq(2,iam), TEMPFAC,
CRSS     $        PIXSIZ_OUT, units)
CRSS      endif

Cji   gather 3Dfft fft_klh in to a full 3D fft map fft3d
Cji   use up to annulus to save memory.
      fft_ann = int(real(nrffta_row)*PIXSIZ_OUT/res_max) + 1
      mem_size = (2*fft_ann+1)*(2*fft_ann+1)*(fft_ann+1)
      allocate(fft3d(mem_size), stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D fft memory allocation failed')
      endif
      mem_total = mem_total + 8 * mem_size
      mem_peak  = max(mem_peak, mem_total)

      call gather3d(fft_klh, hkl_max, fft3d, fft_ann)
      
      deallocate(fft_klh)
      mem_total = mem_total - mem_fft3d
      mem_fft3d = dble(8 * mem_size) 

      call mpi_barrier(mpi_comm_world, mpi_err)
      t1 = wtime()
      t_3dfft = t1 - t2

C **********************************************************************
C *             refine orientation of all input files               *
C **********************************************************************
C
Cji  initialize the CTF mem for half fft_size matrix
C
      mem_size = (nfc_2d_row +1) * (nfc_2d_col * 2 +1)
      allocate(ctf_val(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     abs_ctf(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     disef(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col), 
     +     stat = alloc_stat)
      mem_ctf_val = dble( 12 * mem_size)
      
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +      ' Memory allocation failed for ctf_val, abs_ctf, and disef')
      endif

      mem_total = mem_total + mem_ctf_val
      mem_peak  = max(mem_peak, mem_total)

Cji  loop for all the input parameter files to do cmpt_ort.
      t_analy     = 0
      t_fftanaly    = 0
      total_views = 0
      mem_cmpt_intrps = 0
      
      do i = 1, NFILES

Cji   initialize the parameters for one image file.
         npix_row         =  NBOX(i)
         npix_col         =  NBOX(i)
         max_views        =  NIDS(i)
         angstrom_per_pix = PIXSIZS(i)

         if (iam == 0) then
            pix_file_name    = IMG_FNAMES(i)
            ort_file_name    = PARAM_FNAMES(i)
C     ******PIF_OPEN************
            call PIF_OPEN (pix_file_name, pix_file_unit, 'r')

            call get_1st_ortid(j, ort_file_name)
            call read_1_pif(j, imgprjbig)
            enlargefact = mapprjbig/(imgprjbig*sqrt(real(nrffta_col)))
            print *, 'enlargefact=',enlargefact, imgprjbig, mapprjbig
         endif
         
         call mpi_bcast(enlargefact, 1, mpi_real,
     +        0, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +        'Error in calling MPI_BCAST enlargefact')

C     Initialize arrays containing CTF correction, absolute value of
C     CTF, and radial scaling using Fortran 90 array syntax
         
         ctf_val = 0.0
         abs_ctf = 1.0
         disef   = 1.0
         
         IF (funcweight > 0.0) then 
            call disef_para(disef, nfc_2d_row, nfc_2d_col, funcweight,
     +           angstrom_per_pix, units)
         endif

         IF (CTFMODE > 0 .and. CTFMODE <= 4)  then
            Weight = enlargefact
C                                                          CALL CTF_PARA
C                                                      *****************
Cji using parameter res_max, res_max to avoid soft border for ctfval
C

CRSS Manually set filter and ctfmode so that CTF will be calculated
            filter = 0  !RSS
            ctfmode = 1 !RSS
            weight = 1.0/enlargefact !RSS
            if(iam == 0) write(*,*) 'weight = ', weight !RSS

            call ctf_para(ctf_val, abs_ctf, 
     $           0, nfc_2d_row, -nfc_2d_col, nfc_2d_col, .true.,
     +           CTFMODE, Filter, TEMPFAC, CTF_FF1, CTF_FF2,
     +           PIXSIZS(i), units, VOLTS(i), 
     +           DELF_MAJS(i), DELF_MINS(i), ANG_MAJS(i), AMPFACS(i), 
     +           CS_COEFFS(i), Weight, res_max, res_max)
         endif

Cji read image, and refine its orientation
C                                                       CALL CMPT_INTRPS
C                                                       ****************
         mem_cmpt = 0

         call cmpt_ort(fft3d, hkl_max, fft_ann,
     +        t_analy, t_fftanaly, mem_cmpt, total_views, ctf_val, 
     +        abs_ctf, disef, i)

         mem_cmpt_intrps = max(mem_cmpt_intrps, mem_cmpt)
         mem_peak  = max(mem_peak, mem_total+mem_cmpt)

Cji    finish read the pif file, Close PIF file.
         if(iam == 0) then
            call pif_close (pix_file_unit)
            print *, 'total_view = ', total_views
         endif

      enddo

      deallocate(ctf_val, abs_ctf, disef, fft3d)
      mem_total = mem_total - mem_ctf_val - mem_fft3d

Cji end of all the parameter files

      call mpi_barrier(mpi_comm_world, mpi_err)
      t2 = wtime()
      t_interp = t2 - t1 - t_fftanaly - t_analy

C ***************************************************************************

      if (mem_total .ne. 0) print *,'mem_total=',mem_total,
     +     'At the end, mem_total should be 0!!!'

      if (iam == 0) then
C     Print timing message.

         time = t_init + t_3dfft + t_fftanaly + t_analy + t_interp

         print 40, iam, t_init,
     +        t_3dfft,
     +        t_analy,
     +        t_fftanaly,
     +        t_interp,
     +        time
 40      format (/ 'Node ', i3, 
     +        ' Time used in initialization                      =',
     +        f15.4, ' ' / 8x,
     +        ' Time used in read map & 3DFFT                    =',
     +        f15.4, ' ' / 8x,
     +        ' Time used in read images                         =',
     +        f15.4, ' ' / 8x,
     +        ' Time used in FFT analysis of images              =',
     +        f15.4, ' ' / 8x,
     +        ' Time used in refine orientation & center         =', 
     +        f15.4, ' ' / 8x, 
     +        ' Total time used                                  =',
     +        f15.4)

C     Print memory allocating message.
         write(*,60) iam, mem_peak, mem_map3d, mem_fft3d,
     $        mem_cmpt_intrps
 60      format(/ 'Node ', i3, ' Memory allocation in BYTES:', /
     +        8x, ' Peak memory allocated   =', f18.1 /
     +        8x, ' Memory for 3D Map       =', f18.1 /
     +        8x, ' Memory for 3D FFT       =', f18.1 /
     +        8x, ' Memory used in CMPT_ORT =', f18.1 )

         print *, '**********finished nicely**********'
      endif

      call MPI_FINALIZE ( mpi_err )

      stop
      end program por
